
# q-expansion of newform 980.4.i.q, downloaded from the LMFDB on 17 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 980
weight = 4
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[491, [1, 0]], [197, [1, 0]], [101, [0, -1]]]
aps_data = [[0, 0], [8, -8], [0, -5], [0, 0], [-28, 28], [-82, 0], [-46, 46], [0, 8], [0, 128], [174, 0], [-152, 152], [0, 290], [-50, 0], [396, 0], [0, -296], [570, -570], [-272, 272], [0, -662], [-876, 876], [-880, 0], [-638, 638], [0, 600], [-624, 0], [0, 698], [-754, 0], [354, -354], [0, 1904], [0, -1108], [146, -146], [-2078, 0], [-600, 0], [0, -168], [-394, 394], [472, 0], [0, 2658], [-1256, 1256], [3266, -3266], [0, -1716], [2272, 0], [0, 4226], [-988, 988], [-2138, 0], [0, 3832], [-1378, 1378], [-4266, 0], [5120, -5120], [-1780, 0], [-4896, 0], [-3504, 3504], [0, 826], [0, 278], [4736, 0], [-3990, 3990], [-2296, 0], [0, 4058], [-5640, 5640], [-7758, 7758], [0, 3264], [-566, 566], [4234, 0], [728, -728], [-5490, 0], [-1664, 0], [1688, -1688], [0, -2294], [0, 1426], [0, 740], [-11086, 0], [6420, -6420], [6430, 0], [-5910, 5910], [0, 5720], [8320, -8320], [0, -1718], [364, 0], [0, 2088], [-1446, 1446], [0, -10910], [0, -3090], [2194, -2194], [-3744, 0], [-13106, 0], [-12240, 12240], [-12402, 0], [0, -12280], [0, -3092], [-14750, 0], [0, -6394], [14590, 0], [13664, 0], [0, 4960], [5304, -5304], [21120, -21120], [10500, 0], [0, 3620], [-11176, 0], [0, 3362], [16530, -16530], [0, 6440], [0, 7210], [21796, 0], [-10030, 10030], [9144, -9144], [0, -11946], [-11596, 11596], [1754, -1754], [-13152, 0], [0, -13014], [19936, -19936], [-15578, 0], [0, -3792], [5074, -5074], [-13670, 0], [-18560, 18560], [-8400, 0], [-16770, 16770], [12152, 0], [14896, -14896], [0, -11942], [-31772, 0], [-9502, 9502], [-4750, 0], [0, 23610], [-26852, 26852], [0, -2648], [29118, 0], [0, 14474], [0, 10968], [-18624, 0], [0, -37014], [-1284, 1284], [25584, 0], [0, 3872], [22238, 0], [0, 10178], [-31498, 0], [19090, -19090], [23360, -23360], [28686, 0], [-33226, 33226], [33024, 0], [0, 17090], [11720, -11720], [-34764, 0], [650, -650], [33888, 0], [14246, 0], [362, -362], [0, 25544], [0, -18576], [0, -29606], [-45378, 0], [22540, 0], [0, -20336], [34764, -34764], [-240, 0], [0, -37264], [0, -13286], [13614, 0], [36570, -36570], [0, -8772], [-8742, 0], [-51248, 0], [0, 41376], [-15234, 15234], [-22384, 22384], [3656, -3656], [-15406, 15406]]
